﻿using Kivii.Auth;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Entities;
using Kivii.Seeyon.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Extensions
{
    internal static class ApplyExtension
    {
        public static ResponseToken RequestRestToken(this JsonServiceClient client, string loginName)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var request = new RequestRestToken();
                request.userName = Configs.restUserName;
                request.password = Configs.restPassword;
                request.loginName = loginName;
                var rtns = client.Post<ResponseToken>($"{Configs.RouteRestToken}", request);
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }

        public static Form_InvoiceApply ConvertToForm(this InvoiceApply apply)
        {
            var rtns = new Form_InvoiceApply();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = apply.Kvid.ToString();
            rtns.单据编号 = apply.SerialNumber;
            rtns.日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.所属部门 = KiviiContext.CurrentMember.DepartmentName;
            rtns.经办人 = apply.OperatorName;
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : "4529935671729733794";
            rtns.申请原因 = apply.Remark;
            rtns.数量 = "1";
            rtns.金额 = apply.Amount.ToString();
            return rtns;
        }

        public static ResponseForm<ResponseData> RequestBillForm(this JsonServiceClient client, Form_InvoiceApply form, IAuthSession session)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_InvoiceApply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_InvoiceApply>();
            request.data.templateCode = Configs.templateCode;//"JYHTSQ";
            request.data.draft = "1";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"预借发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_InvoiceApply();
            request.data.data.formmain_0318 = new Form_InvoiceApply();
            request.data.data.formmain_0318 = form;
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }
    }
}
