﻿using Kivii.DataAnnotations;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Extensions;
using Kivii.Linq;
using Kivii.Seeyon;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Entities
{
    [Api(Description = "账单申请")]
    [Alias(Configs.TableNameApply)]
    internal class BillApply : InvoiceApply, IEntityIsSeeyonApply
    {
        public object OnGetApplyResult(Guid Kvid, object Result, string Message)
        {
            if (Kvid == Guid.Empty) throw new Exception("缺少业务编号!");
            if (Result == null) throw new Exception("缺少审批结果信息");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            var apply = conn.SingleById<InvoiceApply>(Kvid);
            if (apply == null) throw new Exception("未找到此业务信息!");
            var result = Result.ToString();
            if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
            EntityLog<InvoiceApply> log = null;
            if (result == "审批通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "流转";
                    log.Summary = "OA审批通过,可进行开票";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            else if (result == "审批不通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.Metadata["SeeyonMessage"] = Message;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "流转";
                    log.Summary = "OA审批不通过,驳回了账单借票审批";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            if (log != null) conn.Insert(log);
            if (!apply.OnlyProperties.IsNullOrEmpty())
            {
                conn.UpdateOnly(apply);
                rtns.Results.Add(apply);
            }

            return rtns;
        }

        public object OnSubmitApplyForm<T>(Guid Kvid)
        {
            Kvid.ThrowIfEmpty("请传入要申请的账单借票信息!");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            InvoiceApply apply = conn.SingleById<InvoiceApply>(Kvid);
            apply.ThrowIfNull("未找到此业务信息!");
            if (apply.Status > (int)InvoiceApplyStatus.ProcessAdoption) throw new Exception("当前账单借票非待审批状态，无法申请审批！");
            var applyForm = apply.ConvertToForm();
            var session = KiviiContext.Request.GetSession();

            var client = new JsonServiceClient(Configs.BaseUrl);
            var resp = client.RequestBillForm(applyForm, session);
            if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
            apply.BizId = resp.data.subject;
            apply.AddOnlyProperties(o => o.BizId);
            apply.BizType = typeof(BillApply).FullName;
            apply.AddOnlyProperties(o => o.BizType);
            apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
            apply.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(apply);
            var _log = new EntityLog<InvoiceApply>();
            _log.OwnerKvid = apply.Kvid;
            _log.Title = "账单借票";
            _log.Type = "流转";
            _log.Summary = $"提交了账单借票,审批将在OA执行";
            conn.Insert(_log);
            var rtns = new RestfulUpdateResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            rtns.Results.Add(apply);

            return rtns;
        }

        public object OnSubmitApplyForm<T>(T form)
        {
            return null;
        }
    }

    public class RequestForm_InvoiceApply
    {
        public Form_InvoiceApply formmain_0318 { get; set; }
    }
    public class Form_InvoiceApply
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public string 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 单据编号 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 日期 { get; set; }

        public string 所属部门 { get; set; }
        public string 经办人 { get; set; }
        public string 付款单位名称 { get; set; }
        public string 纳税人识别号 { get; set; }
        public string 地址电话 { get; set; }
        public string 开户行及账号 { get; set; }

        /// <summary>
        /// 发票类型{普通：4529935671729733794  专票：-4189929216282050282}
        /// </summary>
        public string 发票类型 { get; set; }
        public string 申请原因 { get; set; }
        public string 数量 { get; set; }
        public string 金额 { get; set; }
    }
}
